package net.pricefx.common.apibuilder.calculationflowaction;

import java.math.BigDecimal;
import java.util.ArrayList;

import net.pricefx.common.api.CFTraitType;
import net.pricefx.common.api.CalculationFlowTimeUnit;
import net.pricefx.common.apibuilder.Invocations;

/**
 * CalculationFlowActionBuilder is the starting point of an interface that
 * allows you define actions within calculation flow logic.The calculation flow
 * action builder is available in calculation flow by this call:<br />
 * <code>def builder = actionBuilder</code><br/>
 * Note that you will find full code snippets in the
 * <code>"Command Templates/Calculation Flow Examples"</code> inside calculation
 * flow formulas UI.
 */
public class CalculationFlowActionBuilder implements Help  {

	final Invocations<MethodInvocationRecord> invocations;
	final ArrayList<Action> actions;
	final ScheduleSettings<CalculationFlowActionBuilder> scheduleSettings;

	public CalculationFlowActionBuilder(Invocations<MethodInvocationRecord> invocations, CalculationFlowTimeUnit timeUnit, BigDecimal numberOfTimeUnits, String startingDate, Boolean periodic, CFTraitType traitType) {
		this.invocations = invocations;
		this.actions  = new ArrayList<Action>();
		this.scheduleSettings = new ScheduleSettings<CalculationFlowActionBuilder>(this, invocations, timeUnit, numberOfTimeUnits, startingDate, periodic, traitType);
	}

	public CalculationFlowActionBuilder(Invocations<MethodInvocationRecord> invocations) {
		this(invocations, (CalculationFlowTimeUnit)null, (BigDecimal)null, (String)null, (Boolean)null, (CFTraitType) null);
	}

	/**
	 * Adds CFS action to the list of actions that should be executed.
	 * <br/><br/>
	 * <b>Snippet:</b><br/>
	 * {@code actionBuilder.addCalculatedFieldSetAction('cfsLabel').setCalculate(true)}
	 *
	 * @param cfsLabel label of the CFS as seen in the CFS header level overview in the UI.
	 * @return CFS action object with further configuration options.
	 */
	public CFSAction<CalculationFlowActionBuilder> addCalculatedFieldSetAction(String cfsLabel) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addCalculatedFieldSetAction", cfsLabel));
		CFSAction<CalculationFlowActionBuilder> cfsAction = new CFSAction<CalculationFlowActionBuilder>(this, invocations, cfsLabel);
		actions.add(cfsAction);
		return cfsAction;
	}

	/**
	 * Adds pricelist action to the list of actions that should be executed.
	 * <br/><br/>
	 * <b>Snippet:</b><br/>
	 * {@code actionBuilder.addPricelistAction('pricelistName').setCalculate(true)}
	 *
	 * @param pricelistName name of pricelist as seen in the pricelists header level overview in the UI.
	 * @return pricelist action object with further configuration options.
	 */
	public PricelistAction<CalculationFlowActionBuilder> addPricelistAction(String pricelistName) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addPricelistAction", pricelistName));
		PricelistAction<CalculationFlowActionBuilder> pricelistAction = new PricelistAction<CalculationFlowActionBuilder>(this, invocations, pricelistName);
		actions.add(pricelistAction);
		return pricelistAction;
	}

	/**
	 * Adds manual pricelist action to the list of actions that should be
	 * executed. <br/>
	 * <br/>
	 * <b>Snippet:</b><br/>
	 * {@code actionBuilder.addManualPricelistAction('manualPricelistLabel').setCalculate(true)}
	 *
	 * @param manualPricelistUniqueName
	 *            name of manual pricelist as seen in the manual pricelists
	 *            header level overview in the UI.
	 * @return manual pricelist action object with further configuration
	 *         options.
	 */
	public ManualPricelistAction<CalculationFlowActionBuilder> addManualPricelistAction(String manualPricelistUniqueName) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addManualPricelistAction", manualPricelistUniqueName));
		ManualPricelistAction<CalculationFlowActionBuilder> manualPricelistAction = new ManualPricelistAction<CalculationFlowActionBuilder>(this, invocations, manualPricelistUniqueName);
		actions.add(manualPricelistAction);
		return manualPricelistAction;
	}

	/**
	 * Adds live price grid action to the list of actions that should be
	 * executed. <br/>
	 * <br/>
	 * <b>Snippet:</b><br/>
	 * <ul>
	 * <li>Full recalculation:
	 * {@code actionBuilder.addLivePriceGridAction('livePriceGridLabel').setCalculate(true)}
	 * </li>
	 * <li>Partial recalculation:
	 * {@code actionBuilder.addLivePriceGridAction('livePriceGridLabel').restrictToSkus(['sku1', 'sku2', 'sku3']).setCalculate(true)}
	 * </li>
	 * </ul>
	 *
	 * @param lpgLabel
	 *            label of LPG as seen in the LPG header level overview in the
	 *            UI.
	 * @return LPG action object with further configuration options.
	 */
	public LivePriceGridAction<CalculationFlowActionBuilder> addLivePriceGridAction(String lpgLabel) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addLivePriceGridAction", lpgLabel));
		LivePriceGridAction<CalculationFlowActionBuilder> lpgAction = new LivePriceGridAction<CalculationFlowActionBuilder>(this, invocations, lpgLabel);
		actions.add(lpgAction);
		return lpgAction;
	}

	/**
	 * Adds data load action to the list of actions that should be executed. <br/>
	 * <br/>
	 * <b>Snippet:</b><br/>
	 * <ul>
	 * <li>
	 * {@code actionBuilder.addDataLoadAction('dataLoadLabel', 'dataLoadType', 'target').setCalculate(true)}
	 * </li>
	 * </ul>
	 * <p>Note that dataLoadLabel, dataLoadType and target uniquely identify any data load available in the system.</p>
     * 
     * <p>Valid <code>dataLoadType</code> values are: <code>AddTablePartitions, Calculation, Calendar, Customers,
     * DistributedCalculation, Flush, IndexMaintenance, InternalCopy, ModelCalculation, ModelRecordCalculation,
     * Products, Refresh, Simulation, Truncate.</code></p>
	 * 
	 * @param dataLoadLabel label of data load as seen in the data loads overview in the UI.
	 * @param dataLoadType type of data load as seen in the data loads overview in the UI.
	 * @param target target of data load as seen in the data loads overview in the UI.
	 * @return data load action object with further configuration options.
	 */
	public DataLoadAction<CalculationFlowActionBuilder> addDataLoadAction(String dataLoadLabel, String dataLoadType, String target) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addDataLoadAction", dataLoadLabel, dataLoadType, target));
		DataLoadAction<CalculationFlowActionBuilder> lpgAction = new DataLoadAction<CalculationFlowActionBuilder>(this, invocations, dataLoadLabel, dataLoadType, target);
		actions.add(lpgAction);
		return lpgAction;
	}

	/**
	 * Adds export/archive action to the list of actions that should be
	 * executed.<br/>
	 * <br/>
	 * <b>Snippet:</b><br/>
	 * <ul>
	 * <li>
	 * Export flavour:
	 * {@code actionBuilder.addExportAction('jobName').doExport('templateName')}
	 * </li>
	 * <li>
	 * Archive flovour:
	 * {@code actionBuilder.addExportAction('jobName').doArchive('templateName')}
	 * </li>
	 * </ul>
	 *
	 * @param jobName
	 *            name of the job as seen in "Admin/Import-Export-Archiving" UI.
	 * @return export action object with further configuration options.
	 */
	public ExportAction<CalculationFlowActionBuilder> addExportAction(String jobName) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addExportAction", jobName));
		ExportAction<CalculationFlowActionBuilder> exportAction = new ExportAction<CalculationFlowActionBuilder>(this, invocations, jobName);
		actions.add(exportAction);
		return exportAction;
	}

	/**
	 * Adds simulation action to the list of actions that should be executed.
	 * <br/><br/>
	 * <b>Snippet:</b><br/>
	 * {@code actionBuilder.addSimulationAction('simulationLabel').setCalculate(true)}
	 *
	 * @param simulationLabel name of simulation as seen in the simulations header level overview in the UI.
	 * @return simulation action object with further configuration options.
	 */
	public SimulationAction<CalculationFlowActionBuilder> addSimulationAction(String simulationLabel) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addSimulationAction", simulationLabel));
		SimulationAction<CalculationFlowActionBuilder> simulationAction = new SimulationAction<CalculationFlowActionBuilder>(this, invocations, simulationLabel);
		actions.add(simulationAction);
		return simulationAction;
	}

	/**
	 * Adds rebate record action to the list of actions that should be executed. <br/>
	 * <br/>
	 * <b>Snippet:</b><br/>
	 * {@code actionBuilder.addRebateRecordAction('rrscLabel').setCalculate(true)}
	 *
	 * @param rrscLabel
	 *            rebate record set label as seen in UI
	 * @return rebate record action object with further configuration options.
	 */
	public RebateRecordAction<CalculationFlowActionBuilder> addRebateRecordAction(String rrscLabel) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addRebateRecordAction", rrscLabel));
		RebateRecordAction<CalculationFlowActionBuilder> rebateRecordAction = new RebateRecordAction<CalculationFlowActionBuilder>(this, invocations, rrscLabel);
		actions.add(rebateRecordAction);
		return rebateRecordAction;
	}

	/**
	 * Adds quote mass update action to the list of actions that should be executed. <br/>
	 * <br/>
	 * <b>Snippet:</b><br/>
	 * {@code actionBuilder.addQuoteMassUpdateAction('label').setCalculate(true)}
	 *
	 * @param label
	 * @return quote mass update action object with further configuration options.
	 */
	public QuoteMassUpdateAction<CalculationFlowActionBuilder> addQuoteMassUpdateAction(String label) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addQuoteMassUpdateAction", label));
		QuoteMassUpdateAction<CalculationFlowActionBuilder> quoteMassUpdateAction = new QuoteMassUpdateAction<CalculationFlowActionBuilder>(this, invocations, label);
		actions.add(quoteMassUpdateAction);
		return quoteMassUpdateAction;
	}

	/**
	 * Adds object references action to the list of actions that should be executed. <br/>
	 * <br/>
	 * <b>Snippet:</b><br/>
	 * {@code actionBuilder.addObjRefAction().setFilter(filter).setPartition(partition).setTypeCode('typeCode')}
	 *
	 * @return obj ref action object with further configuration options.
	 */
	public ObjRefAction<CalculationFlowActionBuilder> addObjRefAction() {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".addObjRefAction"));
		ObjRefAction<CalculationFlowActionBuilder> objRefAction = new ObjRefAction<CalculationFlowActionBuilder>(this, invocations);
		actions.add(objRefAction);
		return objRefAction;
	}

	/**
	 *  It's getting you an object that is used to obtain the schedule as seen in the UI + if needed alter it.
	 *
	 * @return schedule settings with further configuration options
	 */
	public ScheduleSettings<CalculationFlowActionBuilder> getScheduleSettings() {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".getScheduleSettings"));
		return scheduleSettings;
	}

	@Override
	public String getObjectName() {
		return "CalculationFlowActionBuilder";
	}

	public ArrayList<Action> getActions() {
		return new ArrayList<Action>(actions);
	}


}
