package net.pricefx.formulaengine.optimization;

import net.pricefx.domain.LookupTable;
import net.pricefx.domain.LookupTable.LookupTableValueType;
import net.pricefx.domain.LookupTableValue;

import java.util.List;
import java.util.Map;

public interface ModelLookupTablesContextReadWrite extends ModelLookupTablesContextReadOnly {
    /**
     * Add a {@link LookupTable} owned by the model. If the table already exists, it is replaced by this
     * new table.
     * <p>
     * The {@link LookupTableValueType} must be one of [MATRIX, MATRIX2, MATRIX3, MATRIX4, MATRIX5, MATRIX6].
     * <p>
     * The definition of columnsSpecs is associating {@link LookupTableValue} fields name with their attribute metadata.
     * Beware of the field names (keys and attributes) may change depending on the {@link LookupTableValueType}.
     * A metadata may not be given for each field,
     * in that case those fields are still present but will be presented to the user in a raw style.
     * <p>
     * Sample code:
     * <pre><code>
     *     model.addLookupTable(
     *             "CustomerDiscounts",
     *             "Discount By Customer",
     *             LookupTableValueType.MATRIX2,
     *             [
     *                 key1: [label:"Customer ID", fieldFormatType: FieldFormatType.TEXT],
     *                 key2: [label:"Customer Group", fieldFormatType: FieldFormatType.TEXT],
     *                 attribute1: [label:"Discount Name", fieldFormatType: FieldFormatType.TEXT],
     *                 attribute2: [label:"Discount Rate", fieldFormatType: FieldFormatType.NUMERIC]
     *             ]
     *     )
     * </code></pre>
     *
     * @param name         The name of the {@link LookupTable}, must be unique per model
     * @param label        The label of the {@link LookupTable}
     * @param valueType    The {@link LookupTableValueType} of the {@link LookupTable}
     * @param columnsSpecs Definition of the {@link LookupTable} value metadata to be created
     * @return The added {@link ModelLookupTable}
     */
    MutableModelLookupTable addLookupTable(String name, String label, LookupTableValueType valueType, Map<String, Map<String, Object>> columnsSpecs);

    /**
     * @param name The name of the {@link LookupTable}, must be unique per model
     * @return The {@link MutableModelLookupTable} owned by this model or {@code null} if it doesn't exist
     */
    @Override
    MutableModelLookupTable lookupTable(String name);

    /**
     * Represents a {@link LookupTable} owned by a ModelObject with mutable operations
     */
    interface MutableModelLookupTable extends ModelLookupTable {
        /**
         * Loads some data in this table
         *
         * @param data     a list of records to add to the table. Each record must associate field names to their value.
         *                 At least all key fields must be given. Any missing entry is considered as a null value.
         * @param truncate If the table should be truncated before loading
         */
        void load(List<Map<String, Object>> data, boolean truncate);

        /**
         * Delete this table
         */
        void drop();

        /**
         * Truncate the content of this table
         */
        void truncate();
    }
}
