package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;

/**
 * Builder class for input type Button {@link InputType#BUTTON}
 */
public class ButtonInputBuilder<VT> extends AbstractInputBuilder<ButtonInputBuilder, VT> {

    protected ButtonInputBuilder(InputBuilderContext builderContext, String paramName) {
        super(builderContext, InputType.BUTTON, paramName);
    }

    /**
     * Set the URL address of the ButtonInputBuilder
     * Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *      .createButtonEntry("Submit")
     *      .setUrl("#startPage")
     *      .setTargetPage("targetPage")
     *      .setTargetPageState("targetPageState")
     *      .setTargetPageComponent("targetPageComponent")
     *      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param url The URL address
     * @return self
     */
    public ButtonInputBuilder setUrl(String url) {
        cp.setURL(url);
        return this;
    }

    /**
     * Set the target page of the ButtonInputBuilder
     * Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *      .createButtonEntry("Submit")
     *      .setUrl("#startPage")
     *      .setTargetPage("targetPage")
     *      .setTargetPageState("targetPageState")
     *      .setTargetPageComponent("targetPageComponent")
     *      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param targetPage Name of the target page
     * @return self
     */
    public ButtonInputBuilder setTargetPage(String targetPage) {
        cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_TARGETPAGE, targetPage);
        return this;
    }

    /**
     * Set the target page state of the ButtonInputBuilder
     * Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *      .createButtonEntry("Submit")
     *      .setUrl("#startPage")
     *      .setTargetPage("targetPage")
     *      .setTargetPageState("targetPageState")
     *      .setTargetPageComponent("targetPageComponent")
     *      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param targetPageState target page state
     * @return self
     */
    public ButtonInputBuilder setTargetPageState(String targetPageState) {
        cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_TARGETPAGESTATE, targetPageState);
        return this;
    }

    /**
     * Set the target page component of the ButtonInputBuilder
     * Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *      .createButtonEntry("Submit")
     *      .setUrl("#startPage")
     *      .setTargetPage("targetPage")
     *      .setTargetPageState("targetPageState")
     *      .setTargetPageComponent("targetPageComponent")
     *      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param targetPageComponent target page component
     * @return self
     */
    public ButtonInputBuilder setTargetPageComponent(String targetPageComponent) {
        cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_TARGETPAGECOMPONENT, targetPageComponent);
        return this;
    }

    @Override
    protected Object _getInput() {
        return null;
    }
}
